@@ -1,28 +1,42 @@
-	subroutine nmixsub(y,n,pw,den,avn,ncmax1,ncd1,ngrid1,ncmax21,
-     &	wtav,muav,sigav,acctry,enttr,ktr,off,partr,nparsamp,devtr,
-c.. input, dimensions (6)
-     &	nmax,ncmax,ncmax2,ncd,ngrid,kkzz,
-c.. input, read only:
-c.. sampling pars (5): 
-     &	nsweep,nburnin,imoves,nstep,kinit,
-c.. switches (13): 
-     &	qempty,qprior,qunif,qfix,qfull,qrkpos,qrange,qkappa,
-     &	qbeta,qpalloc,qpwms,qpclass,qkreg,
-c.. model pars (11): 
-     &	alpha,delta,eee,fff,ggg,hhh,unhw,     
-     &	kappa,lambda,xi,sp,
-c.. output control (4): 
-     &	nspace,nsamp,nskdel,idebug,
-c.. input, but not read-only (2):
-     &	beta,qdebug)
+	subroutine nmixsub(y,n,
+c.. output
+     &	pw,den,avn,wtav,muav,sigav,acctry,
+     &	ifn,lfn,nfn,
+     &	pclass,pz,
+c.. input, dimensions, sampling pars and output control (16)
+     &	nmax,ncmax,ncmax2,ncd,ngrid,k1,k2,kkzz,nsweep,nburnin,
+     &	imoves,nstep,kinit,nspace,idebug,
+c.. switches (1=17): 
+     &	switches,
+c.. model pars (1=12): 
+     &	mpars)
+
+c=     &  switches:
+c=     &	qempty,qprior,qunif,qfix,qfull,qrkpos,qrange,qkappa,
+c=     &	qbeta,qpalloc,qpwms,qpclass,qdebug,
+c=     &  mpars:
+c=     &	alpha,beta,delta,eee,fff,ggg,hhh,unhw,     
+c=     &	kappa,lambda,xi,sp,
 
 c  arguments not (even implicitly) mentioned on Nmix webpage
 c  ncmax,ncmax2,ncd,ngrid,kkzz,
 c  qunif,unhw,
 c  qrkpos,
-c  qkreg,nskdel,
+c  nskdel,
 c  idebug,qdebug
 
+c-- for binary writes
+	use iso_c_binding
+	use f2cio
+	use fef2cio
+
+	integer(c_int) :: nfn,lfn(nfn),ifn(maxval(lfn),nfn)	
+	type(c_ptr):: fp(nfn)
+      integer, parameter :: sng = kind(1.0)
+	real(kind=sng), target :: zr(3)
+	integer, target :: zh(2),zi(1)
+c-- 
+
 	character*6 num,num2
 	character*80 line
 	character moveselect,sep
@@ -30,9 +44,9 @@ c  idebug,qdebug
 	character*8 movabb
 
 	logical qbeta,qkappa,qq,qdebug,qprior,qrange,
-     &		qfull,qfix,qpalloc,
-     &		qunif,qempty,qstop,qpwms,qpclass,
-     &		qkreg,qrkpos
+     &		qfix,qpalloc,qunif,qempty,qstop,qrkpos,
+     &		soutD,soutC,soutA,toutk,toute,toutd,toutp,touta
+	logical switches(17)
 
 	integer hwm,first,free,st1,st2,sp,ckrep
 	integer imoves(nstep),acctry(2,4)
@@ -40,16 +54,14 @@ c  idebug,qdebug
 	real kappa,lambda,mu1,mu2,muc,ms,logratio,logbeta,lprob
 	real kwas,loglikrat,mustar(1)
 
-	INTEGER, PARAMETER :: dbl = KIND(1.0d0)
-	REAL(KIND=dbl) :: dlgama
+	integer, parameter :: dbl = kind(1.0d0)
+	real(kind=dbl) :: dlgama
 
 	real y(nmax),den(ncd,ngrid),pw(ncmax)
 	real wtav(ncmax2,ncmax2),muav(ncmax2,ncmax2),sigav(ncmax2,ncmax2)
-	real avn(ncmax21,ncmax21)
-	real enttr(nsweep/nspace),devtr(nsweep/nspace)
-	real partr(3,nparsamp)
-
-	integer ktr(nsweep/nspace),off(nsweep/nspace),off0
+	real avn(ncmax2,ncmax2)
+	real mpars(12)
+	real pclass(kkzz,ngrid),pz(kkzz,nmax)
 
 	integer split,combine,allocate,weights,parameters,hyper,
      &		birth,death
@@ -61,13 +73,13 @@ c  idebug,qdebug
 
 	integer, dimension(:), allocatable :: start,leng,prev,next,count,
      &	countpos
-	integer, dimension(:), allocatable :: inext,na,z
+	integer, dimension(:), allocatable :: inext,na
+	integer, dimension(:), allocatable, target :: z
 	integer, dimension(:), allocatable :: nfkemp,countden
 
 	real, dimension(:), allocatable :: lp,yv
 	real, dimension(:), allocatable :: wt,mu,mun,bf,avdev,avdevc,pf
 	real, dimension(:), allocatable :: ssq
-	real, dimension(:,:), allocatable :: pclass,pz
 	real, dimension(:), allocatable :: ggh,b,d
 
 	logical, dimension(:), allocatable :: fileopen
@@ -75,30 +87,101 @@ c  idebug,qdebug
 	pi = 4.0*atan(1.0)
 	rr2pi = 1.0/sqrt(2.0*pi)
 
+c.. expand model parameter vector
+
+	alpha = mpars(1)
+	beta = mpars(2)
+	delta = mpars(3)
+	eee = mpars(4)	
+	fff = mpars(5)
+	ggg = mpars(6)
+	hhh = mpars(7)
+	unhw = mpars(8)
+	kappa = mpars(9)
+	lambda = mpars(10)
+	xi = mpars(11)
+	sp = mpars(12)
+
+c.. expand switches vector
+	
+	qempty = switches(1)
+	qprior = switches(2)
+	qunif = switches(3)
+	qfix = switches(4)
+	qrkpos = switches(5)
+	qrange = switches(6)
+	qkappa = switches(7)
+	qbeta = switches(8)
+c.. summary options (den,pcl/scl,avn)
+	soutD = switches(9)
+	soutC = switches(10)
+	soutA = switches(11)
+c.. trace options (k,entropy,deviance,pars,alloc)
+	toutp = switches(12)
+	toutk = switches(13)
+	toutd = switches(14)
+	toute = switches(15)
+	touta = switches(16)
+
+	qdebug = switches(17)
+
 	allocate(start(ncmax),leng(ncmax))
 	allocate(prev(ncmax),next(ncmax))
 	allocate(count(ncmax),countpos(ncmax))
 	allocate(inext(nmax))
 	allocate(na(nmax),z(nmax))
 	allocate(nfkemp(0:9))
-	allocate(countden(ncd1))
+	allocate(countden(ncd))
 
 	allocate(lp(ncmax))
 	allocate(yv(nmax))
 	allocate(wt(ncmax))
 	allocate(mu(ncmax),mun(ncmax))
 	allocate(bf(ncmax))
-	allocate(avdev(ncmax1),avdevc(ncmax1))
+	allocate(avdev(ncmax),avdevc(ncmax))
 	allocate(pf(ncmax))
 	allocate(ssq(ncmax))
 	allocate(fileopen(ncmax))
 
-
-	allocate(pclass(kkzz,ngrid),pz(kkzz,nmax))
-	allocate(ggh(ncd1))
+	allocate(ggh(ncd))
 	allocate(b(ncmax),d(ncmax))
 
-c	call realpr('y',1,y,n)
+c-- for binary writes
+c 1 - a list (pars)
+	if(toutp) then
+		call openabf2cio(ifn(:,1),lfn(1),fp(1),0)
+		zh(1) = 0
+		zh(2) = 0
+		call wif2cio(zh,2,fp(1))
+	end if
+c 2 - integer vector (k)
+	if(toutk) then
+		call openabf2cio(ifn(:,2),lfn(2),fp(2),0)
+		zh(1) = 1
+		zh(2) = 1
+		call wif2cio(zh,2,fp(2))
+	end if
+c 3&4 - real vectors (deviance & entropy)
+	if(toutd) then
+		call openabf2cio(ifn(:,3),lfn(3),fp(3),0)
+		zh(1) = 1
+		zh(2) = 2
+		call wif2cio(zh,2,fp(3))
+	end if
+	if(toute) then
+		call openabf2cio(ifn(:,4),lfn(4),fp(4),0)
+		zh(1) = 1
+		zh(2) = 2
+		call wif2cio(zh,2,fp(4))
+	end if
+c 5 - integer matrix (alloc)
+	if(touta) then
+		call openabf2cio(ifn(:,5),lfn(5),fp(5),0)
+		zh(1) = n
+		zh(2) = 1
+		call wif2cio(zh,2,fp(5))
+	end if
+c-- 
 
 c.. sort out filename stuff
 
@@ -182,8 +265,6 @@ c.. basic statistics
 		ssd = ssd+(y(i)-ysum)**2
 	end do
 
-c	call realpr('ssd',1,ssd,1)
-
 c.. adjust hyperparameters if range-based
 
 	if(qrange) then
@@ -221,83 +302,10 @@ c.. set up prior on k
 
 c.. log parameter values
 
-c	write(0,*) "Run: ",prfx(1:npf)," : ",line(1:ncline)
-c	write(7,*) "Run: ",prfx(1:npf)," : ",line(1:ncline)
-c	write(7,'("random number seed:",i12)') iseed
-c	if(qprior) write(7,'("PRIOR simulation")')
-c	if(qunif) write(7,'("Uniform prior for component means")')
-c	if(qfix) write(7,'("fixed k")')
-c	if(qfull) write(7,'("full output")')
-c	if(qrkpos) write(7,'("reporting kpos option")')
-c	if(qrange) then
-c		write(7,
-c     &	 '("xi      =   ",g11.4,"*R/2+ymid =",g11.4)') xiwas,xi
-c	else
-c		write(7,'("xi      =   ",g11.4)') xi
-c	end if
-c	if(qunif) then
-c		if(qrange) then
-c			write(7,
-c     &	'(" unhw = ",g11.4,"*R/2 =",g11.4)') unhwwas,unhw
-c		else
-c			write(7,'(" unhw = ",g11.4)') unhw
-c		end if
-c	else if(qkappa) then
-c		write(7,'("kappa is Gamma(e,f) with")')
-c		write(7,'("  e     =   ",g11.4)') eee
-c		if(qrange) then
-c			write(7,
-c     &	 '("  f     =   ",g11.4,"*R^2 =",g11.4)') fwas,fff
-c		else
-c			write(7,'("  f     =   ",g11.4)') fff
-c		end if
-c	else
-c		if(qrange) then
-c			write(7,
-c     &	 '("kappa   =   ",g11.4,"/R^2 =",g11.4)') kwas,kappa
-c		else
-c			write(7,'("kappa   =   ",g11.4)') kappa
-c		end if
-c	end if
-c	if(sp.gt.1) write(7,'(" spacing prior, sp=",i3)') sp
-c	write(7,'("alpha   =   ",g11.4)') alpha
-c	if(qbeta) then
-c		write(7,'("beta is Gamma(g,h) with")')
-c		write(7,'("  g     =   ",g11.4)') ggg
-c		if(qrange) then
-c			write(7,
-c     &	 '("  h     =   ",g11.4,"/R^2 =",g11.4)') hwas,hhh
-c		else
-c			write(7,'("  h     =   ",g11.4)') hhh
-c		end if
-c	else
-c		if(qrange) then
-c			write(7,
-c     &	 '("beta   =   ",g11.4,"*R^2 =",g11.4)') bwas,beta
-c		else
-c			write(7,'("beta    =   ",g11.4)') beta
-c		end if
-c	end if
-c	write(7,'("delta   =   ",g11.4)') delta
-c	write(7,'("lambda  =   ",g11.4)') lambda
-c	write(7,'("nsweep  =   ",i8)') nsweep
-c	write(7,'("nstep   =   ",i8)') nstep
-c	write(7,'("nburnin =   ",i8)') nburnin
-c	write(7,'("n       =   ",i8)') n
-c	write(7,'("ymin    =   ",g11.4)') ymin
-c	write(7,'("ymax    =   ",g11.4)') ymax
-
-c	write(7,*) "move schedule: ",moves(1:nstep)
-
 c.. initialise dynamic variables
 c.. initialise number of components, k, and allocation
 
-c	call intpr('kinit',5,kinit,1)
-
 	if(kinit.eq.0) then
-c		open(3,file=base(1:ncbase)//".zin",status='unknown')
-c		read(3,*) (na(i),i=1,n)
-c		close(3)
 		do i = 1,n
 			if(na(i).lt.1.or.na(i).gt.ncmax) return
 		end do
@@ -337,9 +345,7 @@ c		close(3)
 			jq = j
 		    end if
 		end do
-c	write(7,'("initial allocation read from .zin file: k =",i3)') k
 	else
-c	write(7,'("initial number of components:",i4)') kinit
 	if(kinit.eq.1) then
 	    start(1) = 1
 	    leng(1) = n
@@ -383,8 +389,6 @@ c	write(7,'("initial number of components:",i4)') kinit
 	    hwm = kinit
 	end if
 
-c	call intpr('hwm',3,hwm,1)
-
 	kemp = 0
 	j = first
 	do while(j.ne.0)
@@ -392,16 +396,12 @@ c	call intpr('hwm',3,hwm,1)
 	    j = next(j)
 	end do
 
-c	call intpr('kemp',4,kemp,1)
-
 c.. initialise weights
 
 	wtsum = 0.0
 	j = first
 	do while(j.ne.0)
-c	    wt(j) = rgamma(delta+leng(j))
 		call rgamma2(delta+leng(j),wt(j))
-c	call realpr('wt(j)',5,wt(j),1)
 	    wtsum = wtsum+wt(j)
 	    j = next(j)
 	end do
@@ -411,17 +411,10 @@ c	call realpr('wt(j)',5,wt(j),1)
 	    j = next(j)
 	end do
 
-c	call realpr('wtsum',5,wtsum,1)
-
 c.. initialise means
 
 	call gauss(mu,hwm)
 
-c	call realpr('mu',2,mu,hwm)
-
-c	if(qdebug) then
-c		write(18,*) 'mu',(mu(j),j=1,hwm)
-c	end if
 	ssq0 = beta/alpha
 	j = first
 	do while(j.ne.0)
@@ -452,7 +445,6 @@ c	end if
 		con = 1.0/(leng(j)/ssq0+kappa)
 		mu(j) = con*(ysum/ssq0+xi*kappa)+sqrt(con)*mu(j)
 	    end if
-c	call realpr('mu(j)',5,mu(j),1)
 	end if
 	    j = next(j)
 	end do
@@ -462,7 +454,6 @@ c.. initialise variances
 	j = first
 	do while(j.ne.0)
 	if(qprior) then
-c	    ssq(j) = beta/rgamma(alpha)
 		call rgamma2(alpha,temp)
 		ssq(j) = beta/temp
 	else
@@ -472,11 +463,9 @@ c	    ssq(j) = beta/rgamma(alpha)
 		ssd = ssd+(y(i)-mu(j))**2
 		i = inext(i)
 	    end do
-c	    ssq(j) = (beta+0.5*ssd)/rgamma(alpha+0.5*leng(j))
 		call rgamma2(alpha+0.5*leng(j),temp)
 		ssq(j) = (beta+0.5*ssd)/temp
 	end if
-c	call realpr('ssq(j)',6,ssq(j),1)
 
 	j = next(j)
 	end do
@@ -484,13 +473,11 @@ c	call realpr('ssq(j)',6,ssq(j),1)
 c.. now check ordering of mu, and correct
 
 	call reorder(mu,ncmax,next,prev,first)
-c	call realpr('mu',2,mu,hwm)
 
 c.. initialise v
 	do i = 1,n
 		yv(i) = y(i)
 	end do
-c	call realpr('yv',2,yv,n)
 
 c.. initialise accumulators
 
@@ -510,12 +497,14 @@ c.. initialise accumulators
 	end do
 	end do
 
-	if(qfull) then
+	if(soutA) then
 		do j = 1,ncmax2
 		do ij = 1,j
 			avn(j,ij) = 0.0
 		end do
 		end do
+	end if
+	if(soutD) then
 		do krec = 1,ncd
 	    		countden(krec) = 0
 	    		do iyd = 1,ngrid
@@ -531,7 +520,7 @@ c.. initialise accumulators
 		nfkemp(j) = 0
 	end do
 
-	if(qpclass) then
+	if(soutC) then
 	    do iyd = 1,ngrid
 	    do j = 1,kkzz
 		pclass(j,iyd) = 0.0
@@ -561,8 +550,6 @@ c.. initialise accumulators
 	    const1 = 0.5*log(0.5/pi)-sngl(dlgama(dble(alpha)))
 	end if
 	const2 = logbeta(ws,ws)+logbeta(ms,ms)+logbeta(ss,ss)
-c	call realpr('const2',6,const2,1)
-c	if(idebug.lt.0) write(18,'("consts",2g11.5)') const1,const2
 
 	do j = 1,ncmax
 	temp = sp*(j+1)
@@ -571,40 +558,24 @@ c	if(idebug.lt.0) write(18,'("consts",2g11.5)') const1,const2
 	    end do
 	pf(j) = temp
 	end do
-c	call realpr('pf',2,pf,ncmax)
-
-	off0 = 0
-
-c	if(qdebug) then
-c		write(18,'("initialised")')
-c		write(18,*) 'k',k
-c		j = first
-c		do while(j.ne.0)
-c		    write(18,*) 'j,leng,..',j,leng(j),mu(j),ssq(j),wt(j)
-c		    j = next(j)
-c		end do
-c	end if
 
 c.. main loop: MCMC iteration
 
 	do isweep = 1-nburnin,nsweep
 
-c	if(mod(nsweep-isweep,nsweep/10).eq.0) then
-c	    write(0,'(i3,$)') (nsweep-isweep)/(nsweep/10)
-c	end if
+	if(mod(nsweep-isweep,nsweep/10).eq.0) then
+	call countdown((nsweep-isweep)/(nsweep/10))
+	end if
 
 	qdebug = isweep.ge.idebug
 
 	do istep = 0,nstep-1
-c	call intpr('istep',5,istep,1)
 c.. select next move type
 
-c	moveselect = moves(istep+1:istep+1)
 	im = imoves(istep+1)
 	moveselect = movabb(im:im)
 	if(moveselect.eq.'s') then
 	    usw = sdrand(u)
-c	    if(qdebug) write(18,*) 'isweep,usw',isweep,usw
 	    if(usw.lt.b(k)) then
 		move = split
 	    else
@@ -629,8 +600,6 @@ c	    if(qdebug) write(18,*) 'isweep,usw',isweep,usw
 
 	if(move.eq.split) then
 
-c	   write(18,*)'split'
-
 c.. split move ----------------------------------------------
 
 	ntrys = ntrys+1
@@ -638,18 +607,13 @@ c.. split move ----------------------------------------------
 20	j = 1+int(hwm*sdrand(u))
 	if(prev(j).lt.0) go to 20
 
-c	if(qdebug) write(18,*) 'split:isweep,j',isweep,j
-
 	wtc = wt(j)
 	muc = mu(j)
 	ssqc = ssq(j)
 
-c	u1 = rbeta(ws,ws)
 	call rbeta2(ws,ws,u1)
 	cu1 = 1.0-u1
-c	u2 = rbeta(ms,ms)
 	call rbeta2(ms,ms,u2)
-c	if(qdebug) write(18,*) 'split:isweep,u1,u2',isweep,u1,u2
 	wt1 = wtc*u1
 	wt2 = wtc-wt1
 	mu1 = muc-sqrt(ssqc*wt1*wt2)*u2/wt1
@@ -659,7 +623,6 @@ c.. check mu in order: else reject
 
 	j1 = prev(j)
 	if(j1.ne.0) then
-c		if(qdebug) write(18,*) 'mu1,mu(j1)',mu1,mu(j1)
 		if(mu(j1).gt.mu1) then
 			nrejr = nrejr+1
 			go to 24
@@ -669,7 +632,6 @@ c		if(qdebug) write(18,*) 'mu1,mu(j1)',mu1,mu(j1)
 	end if
 	j1 = next(j)
 	if(j1.ne.0) then
-c		if(qdebug) write(18,*) 'mu2,mu(j1)',mu2,mu(j1)
 		if(mu(j1).lt.mu2) then
 			nrejr = nrejr+1
 			go to 24
@@ -678,9 +640,7 @@ c		if(qdebug) write(18,*) 'mu2,mu(j1)',mu2,mu(j1)
 		if(mu2.gt.xi+unhw) go to 24
 	end if
 
-c	u3 = rbeta(ss,ss)
 	call rbeta2(ss,ss,u3)
-c	if(qdebug) write(18,*) 'isweep,u3',isweep,u3
 	cu3 = 1.0-u3
 	temp = wtc*ssqc*(1.0-u2**2)
 	ssq1 = temp*u3/wt1
@@ -718,8 +678,6 @@ c.. allocate observations in component to be split
 	    i = in
 	end do
 
-c	if(qdebug) write(18,*) 'isweep,l1,l2',isweep,l1,l2
-
 	klow = k
 
 c.. compute ratio and decide
@@ -740,20 +698,12 @@ c.. compute ratio and decide
 	end do
 	end if
 
-c	if(qdebug) then
-c		write(18,*) 'split1:isw,move,logr',isweep,move,logratio
-c	end if
-
 c.. p(k,w,z) terms
 
 	logratio = logratio+(lp(klow+1)-lp(klow))
      &		+(delta-1.0)*log(wt1*wt2/wtc)-logbeta(delta,klow*delta)
      &		+l1*log(wt1/wtc)+l2*log(wt2/wtc)
 
-c	if(qdebug) then
-c		write(18,*) 'split2:isw,move,logr',isweep,move,logratio
-c	end if
-
 c.. p(theta) terms
 
 	if(qunif) then
@@ -768,14 +718,6 @@ c.. p(theta) terms
      &		-(alpha+1.0)*log(ssq1*ssq2/ssqc)
      &		-beta*(1.0/ssq1+1.0/ssq2-1.0/ssqc)
      &		+log(pf(klow))
-c	if(qdebug) then
-c		write(18,*) const1,alpha,beta,kappa,ssq1,ssq2,ssqc
-c		write(18,*) const1+alpha*log(beta)+0.5*log(kappa)
-c		write(18,*) -0.5*kappa*((mu1-xi)**2+(mu2-xi)**2-(muc-xi)**2)
-c		write(18,*) -(alpha+1.0)*log(ssq1*ssq2/ssqc)
-c		write(18,*) -beta*(1.0/ssq1+1.0/ssq2-1.0/ssqc)
-c		write(18,*) +log(pf(klow))
-c	end if
 	    if(sp.gt.1) then
 		sqrk = sqrt(kappa)
 		jplus = next(j)
@@ -804,42 +746,26 @@ c	end if
 	    end if
 	end if
 
-c	if(qdebug) then
-c		write(18,*) 'split3:isw,move,logr',isweep,move,logratio
-c	end if
-
 c.. proposal terms
 
-
 	logratio = logratio+const2
      &		+log(d(klow+1)/b(klow))-lprob
      &		-(ws-1.0)*log(u1*cu1)
      &		-(ms-1.0)*log(u2*(1.0-u2))
      &		-(ss-1.0)*log(u3*cu3)
 
-c	if(qdebug) then
-c		write(18,*) 'split4:isw,move,logr',isweep,move,logratio
-c	end if
-
 c.. Jacobian terms
 
 	logratio = logratio
      &		+log(wtc*abs(mu1-mu2)*ssq1*ssq2/ssqc)
      &		-log(u2*(1.0-u2**2)*u3*cu3)
 
-c	if(qdebug) then
-c		write(18,*) 'split5:isw,move,logr',isweep,move,logratio
-c	end if
-
 	logratio = max(-20.0,min(20.0,logratio))
 
       if(sdrand(u).lt.exp(logratio)) then
 
 c.. accept split
 
-c		if(qdebug) then
-c		    write(18,'("split accepted")')
-c		end if
 		naccs = naccs+1
 
 		if(free.eq.0) return
@@ -869,8 +795,6 @@ c		end if
 		if(l1*l2.eq.0) kemp = kemp+1
 		kpos = k-kemp
 
-c		if(.not.qkreg.and..not.qfix) write(8,'(2i3,i8)') k,kpos,isweep
-
 	else
 
 	    if(l1.ne.0) then
@@ -886,8 +810,6 @@ c		if(.not.qkreg.and..not.qfix) write(8,'(2i3,i8)') k,kpos,isweep
 
 	else if(move.eq.combine) then
 
-c	   write(18,*)'combine'
-
 c.. combine move ----------------------------------------------
 
 	ntryc = ntryc+1
@@ -912,21 +834,15 @@ c.. combine move ----------------------------------------------
 	ssq2 = ssq(j2)
 	wtc = wt1+wt2
 	muc = (wt1*mu1+wt2*mu2)/wtc
-c	ssqc = (wt1*(mu1**2+ssq1)
-c     &		+wt2*(mu2**2+ssq2))/wtc-muc**2
 	ssqc = wt1*wt2*((mu1-mu2)/wtc)**2 +(wt1*ssq1+wt2*ssq2)/wtc
 	u1 = wt1/wtc
 	cu1 = wt2/wtc
-c	u2 = (muc-mu1)*wt1/sqrt(ssqc*wt1*wt2)
 	u2 = (mu2-mu1)*sqrt(wt1*wt2/ssqc)/wtc
 	u2 = max(u2,1e-12)
 	u2 = min(u2,1.0-1e-4)
 	u3 = wt1*ssq1/(wt1*ssq1+wt2*ssq2)
 	cu3 = wt2*ssq2/(wt1*ssq1+wt2*ssq2)
 
-c	write(18,'(3f8.5)')wt1,wt2,wtc,mu1,mu2,muc,ssq1,ssq2,ssqc,u1,u2,u3
-c	write(18,*)ssq1,ssq2,ssqc,u2
-
 	lprob = 0.0
 	i = st1
 	do while(i.ne.0)
@@ -976,20 +892,12 @@ c.. compute ratio and decide
 
 	loglikrat = logratio
 
-c	if(qdebug) then
-c		write(18,'(2i6,1pe16.7)') isweep,move,logratio
-c	end if
-
 c.. p(k,w,z) terms
 
 	logratio = logratio+(lp(klow+1)-lp(klow))
      &		+(delta-1.0)*log(wt1*wt2/wtc)-logbeta(delta,klow*delta)
      &		+l1*log(wt1/wtc)+l2*log(wt2/wtc)
 
-c	if(qdebug) then
-c		write(18,'(2i6,1pe16.7)') isweep,move,logratio
-c	end if
-
 c.. p(theta) terms
 
 	if(qunif) then
@@ -1032,11 +940,6 @@ c.. p(theta) terms
 	    end if
 	end if
 
-c	if(qdebug) then
-c		write(18,'(2i6,1pe16.7)') isweep,move,logratio
-c	end if
-
-
 c.. proposal terms
 
 	logratio = logratio+const2
@@ -1045,35 +948,20 @@ c.. proposal terms
      &		-(ms-1.0)*log(u2*(1.0-u2))
      &		-(ss-1.0)*log(u3*cu3)
 
-c	if(qdebug) then
-c		write(18,'(2i6,1pe16.7)') isweep,move,logratio
-c	end if
-
-
 c.. Jacobian terms
 
 	logratio = logratio
      &		+log(wtc*abs(mu1-mu2)*ssq1*ssq2/ssqc)
      &		-log(u2*(1.0-u2**2)*u3*cu3)
 
-c	if(qdebug) then
-c		write(18,'(2i6,1pe16.7)') isweep,move,logratio
-c	end if
-
 	logratio = max(-20.0,min(20.0,logratio))
 
-
 	if(sdrand(u).lt.exp(-logratio)) then
 
 c.. accept combine
 
-c		if(qdebug) then
-c		    write(18,'("combine accepted")')
-c		end if
 	    naccc = naccc+1
 
-c 	    next(j1) = next(j2)
-c	    if(next(j1).ne.0) prev(next(j1)) = j1
 	if(prev(j2).ne.0) then
 		next(prev(j2)) = next(j2)
 	else
@@ -1098,15 +986,11 @@ c	    if(next(j1).ne.0) prev(next(j1)) = j1
 	    if(l1*l2.eq.0) kemp = kemp-1
 	    kpos = k-kemp
 
-c		if(.not.qkreg.and..not.qfix) write(8,'(2i3,i8)') k,kpos,isweep
-
 	end if
 
 
 	else if(move.eq.allocate) then
 
-c	   write(18,*)'alloc'
-
 c.. Gibbs move for allocation -------------------------------
 
 	if(k.gt.1) then
@@ -1120,32 +1004,15 @@ c.. Gibbs move for allocation -------------------------------
 	      j = next(j)
 	   end do
 
-c	   if(qdebug) then
-c	      write(18,'("allocation move")')
-c	      j = first
-c	      do while(j.ne.0)
-c		 write(18,'(2i6)') j,leng(j)
-c		 j = next(j)
-c	      end do
-c	   end if
-	   
-
-	   
 	end if
 	
 	else if(move.eq.weights) then
 
-c	   write(18,*)'weights'
-
 c.. Gibbs move for weights  ---------------------------------
 
-c	if(qdebug) then
-c		write(18,'("weights move")')
-c	end if
 	wtsum = 0.0
 	j = first
 	do while(j.ne.0)
-c	    wt(j) = rgamma(delta+leng(j))
 	    call rgamma2(delta+leng(j),wt(j))
 	    wtsum = wtsum+wt(j)
 	    j = next(j)
@@ -1153,18 +1020,13 @@ c	    wt(j) = rgamma(delta+leng(j))
 	j = first
 	do while(j.ne.0)
 	    wt(j) = wt(j)/wtsum
-c	    if(qdebug) write(18,'(i4,f10.4)') j,wt(j)
 	    j = next(j)
 	end do
 
-
 	else if(move.eq.parameters) then
 
-c	   write(18,*)'params'
-
 c.. Metropolis and/or Gibbs moves for component parameters --
 
-
 	call gauss(mun,hwm)
 	j = first
 	do while(j.ne.0)
@@ -1238,21 +1100,12 @@ c.. copy over if accepted
 	   mu(j) = mun(j)
 	   j = next(j)
 	end do
-	
 
- 66	if(qdebug) then
-c		write(18,'("means updated")')
-c		j = first
-c		do while(j.ne.0)
-c		    write(18,'(i4,f10.4)') j,mu(j)
-c		    j = next(j)
-c		end do
-	end if
+ 66	continue
 
 	j = first
 	do while(j.ne.0)
 	if(qprior) then
-c	    ssq(j) = beta/rgamma(alpha)
 		call rgamma2(alpha,temp)
 		ssq(j) = beta/temp
 	else
@@ -1262,7 +1115,6 @@ c	    ssq(j) = beta/rgamma(alpha)
 		ssd = ssd+(yv(i)-mu(j))**2
 		i = inext(i)
 	    end do
-c	    ssq(j) = (beta+0.5*ssd)/rgamma(alpha+0.5*leng(j))
 		call rgamma2(alpha+0.5*leng(j),temp)
 		ssq(j) = (beta+0.5*ssd)/temp
 	end if
@@ -1270,19 +1122,8 @@ c	    ssq(j) = (beta+0.5*ssd)/rgamma(alpha+0.5*leng(j))
 	j = next(j)
 	end do
 
-c	if(qdebug) then
-c		write(18,'("variances updated")')
-c		j = first
-c		do while(j.ne.0)
-c		    write(18,'(i4,f10.4)') j,ssq(j)
-c		    j = next(j)
-c		end do
-c	end if
-
 	else if(move.eq.hyper) then
 
-c	   write(18,*)'hyper'
-
 c.. Gibbs move for hyperparameters --------------------------
 
 	if(qbeta) then
@@ -1292,7 +1133,6 @@ c.. Gibbs move for hyperparameters --------------------------
 			sum = sum+1.0/ssq(j)
 			j = next(j)
 		end do
-c		beta = rgamma(ggg+k*alpha)/(hhh+sum)
 		call rgamma2(ggg+k*alpha,temp)
 		beta = temp/(hhh+sum)
 	end if
@@ -1303,7 +1143,6 @@ c		beta = rgamma(ggg+k*alpha)/(hhh+sum)
 			sum = sum+(mu(j)-xi)**2
 			j = next(j)
 		end do
-c		kappa = rgamma(eee+0.5*k)/(fff+0.5*sum)
 		call rgamma2(eee+0.5*k,temp)
 		kappa = temp/(fff+0.5*sum)
 	end if
@@ -1313,8 +1152,6 @@ c.. Birth of an empty component
 
 	else if(move.eq.birth) then
 
-c	   write(18,*)'birth'
-
 	    ntryb = ntryb+1
 
 c.. compute logratio
@@ -1322,10 +1159,7 @@ c.. compute logratio
 	    klow = k
 	    kemplow = kemp
 
-c	    wtstar = rbeta(1.0,real(klow))
 		call rbeta2(1.0,real(klow),wtstar)
-
-c	    ssqstar = beta/rgamma(alpha)
 		call rgamma2(alpha,temp)
 		ssqstar = beta/temp
 
@@ -1396,8 +1230,6 @@ c.. Jacobian terms
 		kemp = kemp+1
 		kpos = k-kemp
 
-c		if(.not.qkreg.and..not.qfix) write(8,'(2i3,i8)') k,kpos,isweep
-
 		naccb = naccb+1
 
 	    end if
@@ -1406,8 +1238,6 @@ c.. Death of an empty component
 
 	else if(move.eq.death) then
 
-c	   write(18,*)'death'
-
 	    ntryd = ntryd+1
 
 	    if(kemp.gt.0) then
@@ -1464,8 +1294,6 @@ c.. Jacobian terms
 			kemp = kemp-1
 			kpos = k-kemp
 
-c			if(.not.qkreg.and..not.qfix) write(8,'(2i3,i8)') k,kpos,isweep
-
 			naccd = naccd+1
 
 		end if
@@ -1481,8 +1309,6 @@ c.. Trap for illegal move
 
 	end if
 
-
-
 c.. end of 'istep' loop:
 	end do
 
@@ -1490,16 +1316,10 @@ c.. logging stage
 
 	kpos = k-kemp
 
-c	if(qkreg.and..not.qfix) then
-c	    if(mod(isweep,nskdel).eq.0) then
-c		write(8,'(2i3,i8)') k,kpos,isweep
-c	    end if
-c	end if
-
 	if(isweep.gt.0) then
 
-	count(k) = count(k)+1
-	countpos(kpos) = countpos(kpos)+1
+	if(k.le.ncmax) count(k) = count(k)+1
+	if(kpos.le.ncmax) countpos(kpos) = countpos(kpos)+1
 
 c.. updates complete: record information
 
@@ -1508,7 +1328,7 @@ c.. updates complete: record information
 	else
 		krep = k
 	end if
-	if(krep.le.10) then
+	if(krep.le.ncmax2) then
 		j = first
 		ij = 1
 		do while(j.ne.0)
@@ -1516,14 +1336,14 @@ c.. updates complete: record information
 			wtav(krep,ij) = wtav(krep,ij)+wt(j)
 			muav(krep,ij) = muav(krep,ij)+mu(j)
 			sigav(krep,ij) = sigav(krep,ij)+sqrt(ssq(j))
-			if(qfull) avn(krep,ij) = avn(krep,ij)+leng(j)
+			if(soutA) avn(krep,ij) = avn(krep,ij)+leng(j)
 			ij = ij+1
 		    end if
 		    j = next(j)
 		end do
 	end if
 
-	if((.not.qprior).and.qfull) then
+	if((.not.qprior).and.toutd) then
 
 	    dev = 0.0
 	    devc = 0.0
@@ -1545,7 +1365,6 @@ c.. updates complete: record information
 	    end do
 	    dev = -2.0*dev
 	    devc = -2.0*devc
-c	    devc = dev+devcorrection
 
 	    avdev(krep) = avdev(krep)+dev
 	    avdevc(krep) = avdevc(krep)+devc
@@ -1556,36 +1375,16 @@ c	    devc = dev+devcorrection
 
 	ent = entropy(n,leng,ncmax,first,next)
 
-c	if(qbeta.or.qkappa) write(10,*) beta,kappa,k,kpos
-
-c	if(qfull) write(9,'(2i4,2f8.2)') k,kpos,dev,devc
-
-	if(krep.le.10.and.qfull.and.(qpwms.or.qpalloc)) then
-c	    if(.not.fileopen(krep)) then
-c		istd2 = 6-int(log10(0.5+krep))
-c		write(num2,'(i6)') krep
-c		if(qpwms) then
-c		open(25+krep,file=
-c     &			prfx(1:npf)//".wms."//num2(istd2:6),
-c     &                  status='unknown')
-c		end if
-c		if(qpalloc) then
-c		open(25+ncmax+krep,file=
-c     &			prfx(1:npf)//".z."//num2(istd2:6),
-c     &                  status='unknown')
-c		end if
-c		fileopen(krep) = .true.
+c	if(krep.le.10.and.qfull.and.(qpwms.or.qpalloc)) then
+c	    if(qpwms) then
+c	        j = first
+c		do while(j.ne.0)
+c		    j = next(j)
+c		end do
 c	    end if
-	    if(qpwms) then
-	        j = first
-		do while(j.ne.0)
-c		    if((.not.qrkpos).or.leng(j).gt.0) then
-c			write(25+krep,'(3f12.5)') wt(j),mu(j),sqrt(ssq(j))
-c		    end if
-		    j = next(j)
-		end do
-	    end if
-	    if(qpalloc) then
+c	end if
+
+	if(touta) then
 		j = first
 		ij = 1
 		do while(j.ne.0)
@@ -1597,8 +1396,9 @@ c		    end if
 			j = next(j)
 			ij = ij+1
 		end do
-c		write(25+ncmax+krep,'(20i3)') (z(i),i=1,n)
-	    end if
+c-- binary write allocations
+		call wif2cio(z,n,fp(5))
+c--
 	end if
 
 	end if
@@ -1609,37 +1409,45 @@ c		write(25+ncmax+krep,'(20i3)') (z(i),i=1,n)
 	if(kemp.le.9) nfkemp(kemp) = nfkemp(kemp)+1
 
 	if(mod(isweep,nspace).eq.0) then
-c		write(17,*) k,kpos,ent
-		enttr(isweep/nspace) = ent
-		ktr(isweep/nspace) = k
-		off(isweep/nspace) = off0
-		if(off0+k.le.nparsamp) then
-			j = first
-			do while(j.ne.0)
-				off0 = off0+1
-				partr(1,off0) = wt(j)
-				partr(2,off0) = mu(j)
-				partr(3,off0) = sqrt(ssq(j))
-				j = next(j)
-			end do
-		end if
-		if(qfull) devtr(isweep/nspace) = dev
-	end if
 
-	if(mod(isweep,max(1,nsweep/nsamp)).eq.0) then
+c-- binary writes to trace file: (1) 3k 4-byte reals (pars)
 
-c	write(4,'(2i3)') k,kpos
+	if(toutp) then
+	zh(1) = k*3
+	zh(2) = 2
+	call wif2cio(zh,2,fp(1))
 	j = first
 	do while(j.ne.0)
-	    stdev = sqrt(ssq(j))
-c	    write(4,'(3f12.5,i4)') wt(j),mu(j),stdev,leng(j)
-	    j = next(j)
+		zr(1) = wt(j)
+		zr(2) = mu(j)
+		zr(3) = sqrt(ssq(j))
+		call wrf2cio(zr,3,fp(1))
+		j = next(j)
 	end do
 	end if
 
+c (2) 1 integer (k)
+
+	if(toutk) then
+	zi(1) = k
+	call wif2cio(zi,1,fp(2))
+	end if
+
+c (3&4) 1 4-byte real (dev and ent)
+	if(toutd) then
+		zr(1) = dev
+		call wrf2cio(zr,1,fp(3))
+	end if
+	if(toute) then
+		zr(1) = ent
+		call wrf2cio(zr,1,fp(4))
+	end if
+c-- 
+	end if
+
 c.. accumulate mean densities
 
-	if(qfull) then
+	if(soutD.or.soutC) then
 	    krec = min(k,ncd)
 	    countden(krec) = countden(krec)+1
 	    do iyd = 1,ngrid
@@ -1663,7 +1471,7 @@ c.. accumulate mean densities
 		    j = next(j)
 		    ij = ij+1
 		end do
-		if(qpclass) then
+		if(soutC) then
 		    if(k.ge.k1.and.k.le.k2) then
 		    ko = ((k+k1-3)*(k-k1))/2
 		    do ij = 1,k-1
@@ -1674,7 +1482,7 @@ c.. accumulate mean densities
 		end if
 	    end do
 
-		if(qpclass) then
+		if(soutC) then
 		if(k.ge.k1.and.k.le.k2) then
 		    ko = ((k+k1-3)*(k-k1))/2
 		    j = first
@@ -1707,19 +1515,13 @@ c.. end of main loop
 		end do
 	end if
 
-c	if(qdebug.and.mod(isweep,nsweep/50).eq.0) write(18,*) isweep
-
 	end do
 
-c	write(0,*)
-
-c	write(7,'("splits:  ",i6," out of",i7)') naccs,ntrys
-c	write(7,'("split rej r:  ",i6)') nrejr
-c	write(7,'("combines:",i6," out of",i7)') naccc,ntryc
-c	if(qempty) then
-c		write(7,'("births:  ",i6," out of",i7)') naccb,ntryb
-c		write(7,'("deaths:  ",i6," out of",i7)') naccd,ntryd
-c	end if
+c-- for binary writes
+	do j = 1,nfn
+		if(switches(11+j)) call closef2cio(fp(j))
+	end do
+c-- 
 
 	acctry(1,1) = naccs
 	acctry(2,1) = ntrys
@@ -1737,63 +1539,25 @@ c.. output posterior probs for k, and Bayes factors
 		pw(k) = real(count(k))/nsweep
 		if(kbase.eq.0.and.count(k).ne.0) kbase = k
 	end do
-c	write(12,'(5f16.6)') pw
 
 	do k = 1,ncmax
 	    bf(k) = (pw(k)/pw(kbase))/exp(lp(k)-lp(kbase))
 	end do
 
-c	write(12,'(1x)')
-c	write(12,'(5f16.6)') bf
-
 c.. write prior probs p(k)
 
 	do k = 1,ncmax
 	    pw(k) = exp(lp(k))
 	end do
-c	write(12,'(1x)')
-c	write(12,'(5f16.6)') pw
 
 c.. write posterior probs for number of nonempty components
 
 	do k = 1,ncmax
 		   pw(k) = real(countpos(k))/nsweep
 	end do
-c	write(12,'(1x)')
-c	write(12,'(5f16.6)') pw
 
 	avkemp = avkemp/nkemp
 	ppkemp = ppkemp/nkemp
-c	write(12,'(2f10.4)') avkemp,ppkemp
-c	write(12,'(10i8)') nfkemp
-
-	if(qfull) then
-
-	    avdevall = 0.0
-	    avdevcall = 0.0
-	    do k = 1,ncmax
-		avdevall = avdevall+avdev(k)
-		avdevcall = avdevcall+avdevc(k)
-	    end do
-	    avdevall = avdevall/nsweep
-	    avdevcall = avdevcall/nsweep
-
-	    if(qrkpos) then
-		do k = 1,ncmax
-		    avdev(k) = avdev(k)/max(1,countpos(k))
-		    avdevc(k) = avdevc(k)/max(1,countpos(k))
-		end do
-	    else
-		do k = 1,ncmax
-		    avdev(k) = avdev(k)/max(1,count(k))
-		    avdevc(k) = avdevc(k)/max(1,count(k))
-		end do
-	    end if
-c	    write(12,'(1x)')
-c	    write(12,'(5f16.6)') avdev
-c	    write(12,'(1x)')
-c	    write(12,'(5f16.6)') avdevc
-	end if
 
 c.. output posterior expectations of parameters
 
@@ -1803,60 +1567,44 @@ c.. output posterior expectations of parameters
 	else
 		ckrep = count(k)
 	end if
-c	write(11,'(i3,i8)') k,ckrep
 	ckrep = max(1,ckrep)
 	do ij = 1,k
 		wtav(k,ij) = wtav(k,ij)/ckrep
 		muav(k,ij) = muav(k,ij)/ckrep
 		sigav(k,ij) = sigav(k,ij)/ckrep
-c		write(11,'(3f12.5)') wtav(k,ij),muav(k,ij),sigav(k,ij)
 	end do
 	end do
 
 c.. output average group sizes
 
-	if(qfull) then
+	if(soutA) then
 	    do k = 1,ncmax2
 		if(qrkpos) then
 		    ckrep = countpos(k)
 		else
 		    ckrep = count(k)
 		end if
-c		write(16,'(2i8)') k,ckrep
 		ckrep = max(1,ckrep)
 		do ij = 1,k
 		    avn(k,ij) = avn(k,ij)/ckrep
 		end do
-c		write(16,'(10f8.2)') (avn(k,ij),ij=1,k)
 	    end do
 	end if
 c.. output densities
 
-c	do iyd=1,6
-c	call realpr('den(,iyd)',9,den(1,iyd),7)
-c	end do
-
-	if(qfull) then
+	if(soutD) then
 	    do k = 1,ncd-1
 	    countden(ncd) = countden(ncd)+countden(k)
 	    do iyd = 1,ngrid
 	    den(ncd,iyd) = den(ncd,iyd)+den(k,iyd)
 	    end do
 	    end do
-c	    if(qfull)then
-c		write(13,*) (yd0+iyd*ydinc,iyd=1,ngrid)
-c	    end if
 	    do k = 1,ncd
 	    do iyd = 1,ngrid
 	    den(k,iyd) = den(k,iyd)/max(countden(k),1)
 	    end do
-c	    if(qfull)then
-c		write(13,*) (den(k,iyd),iyd=1,ngrid)
-c	    end if
 	    end do
 
-c	    write(12,'("     D-hat:",g13.5)') avdevall
-c	    write(12,'(" new D-hat:",g13.5)') avdevcall
 	    do k = 1,ncd
 		ggh(k) = 0.0
 		if(countden(k).gt.0) then
@@ -1867,20 +1615,15 @@ c	    write(12,'(" new D-hat:",g13.5)') avdevcall
 		  ggh(k) = -2.0*ggh(k)
 		end if
 	    end do
-c	    write(12,'("  G(g-hat):",g13.5)') ggh(ncd)
-c	    write(12,'("G(g-hat_k):",3g13.5/(11x,3g13.5))')
-c     &		(ggh(k),k=1,ncd-1)
 
 	end if
 
 c.. output predictive classification
 
-	if(qfull.and.qpclass) then
+	if(soutC) then
 
-c	write(14,*) (yd0+iyd*ydinc,iyd=1,ngrid)
 	do k = k1,k2
 	    ko = ((k+k1-3)*(k-k1))/2
-c	    write(14,'(i4)') k
 	    do ij = 1,k-1
 		do iyd = 1,ngrid
 		    pclass(ko+ij,iyd) = pclass(ko+ij,iyd)/max(1,count(k))
@@ -1891,7 +1634,6 @@ c	    write(14,'(i4)') k
      &				pclass(ko+ij-1,iyd)
 		    end do
 		end if
-c	    write(14,'(10f8.4)') (pclass(ko+ij,iyd),iyd=1,ngrid)
 	    end do
 	end do
 
@@ -1899,7 +1641,6 @@ c.. output within-sample classification
 
 	do k = k1,k2
 	    ko = ((k+k1-3)*(k-k1))/2
-c	    write(15,'(i4)') k
 	    do ij = 1,k-1
 		do i = 1,n
 		    pz(ko+ij,i) = pz(ko+ij,i)/max(1,count(k))
@@ -1909,7 +1650,6 @@ c	    write(15,'(i4)') k
 			pz(ko+ij,i) = pz(ko+ij,i)+pz(ko+ij-1,i)
 		    end do
 		end if
-c		write(15,'(10f8.4)') (pz(ko+ij,i),i=1,n)
 	    end do
 	end do
 
@@ -1918,3 +1658,5 @@ c		write(15,'(10f8.4)') (pz(ko+ij,i),i=1,n)
 	return
 	end
 
+
+
